<?php

namespace Ymina\Balanceo\Helper;

use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\ScopeInterface;
use Magento\Catalog\Model\ProductFactory;

class Data extends AbstractHelper
{

    const XML_PATH_SHOW_BALANCEO = 'checkout/options/enabled_balanceo';
    const XML_PATH_TITLE_BALANCEO = 'checkout/options/label_balanceo';
    const XML_PATH_AMOUNT_BALANCEO = 'checkout/options/amaunt_balanceo';

    /**
     * @var ProductRepository
     */
    private $_productRepository;

    protected $_productFactory;

    public function __construct(
        Context           $context,
        ProductRepository $productRepository,
        ProductFactory $productFactory
    )
    {

        $this->_productRepository = $productRepository;
        $this->_productFactory = $productFactory;
        parent::__construct($context);
    }

    /**
     * @return int
     */
    public function isEnabledTaxBalanseo()
    {
        return (int)$this->scopeConfig->getValue(self::XML_PATH_SHOW_BALANCEO, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getTaxBalanseoTitle()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_TITLE_BALANCEO, ScopeInterface::SCOPE_STORE);
    }


    /**
     * @return float
     */
    public function getTaxBalanseoAmount()
    {
        return (float)$this->scopeConfig->getValue(self::XML_PATH_AMOUNT_BALANCEO, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $sku
     * @return \Magento\Catalog\Api\Data\ProductInterface|\Magento\Catalog\Model\Product|null
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductBySku($sku)
    {
        return $this->_productRepository->get($sku);
    }

    /**
     * @param $id
     * @return mixed
     */
    public function getProductById($id) {
        return $this->_productFactory->create()->load($id);
    }

    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @return bool
     */
    public function isValid(\Magento\Quote\Model\Quote $quote, $items)
    {

        if (!$this->isEnabledTaxBalanseo()) {
            return false;
        }

        $method = $quote->getShippingAddress()->getShippingMethod();
        if ($method !== "amstorepickup_amstorepickup") {
           return false;
        }

        if (!$items) {
            return false;
        }

        $continue = false;

        foreach ($items as $item) {
            $product = $this->getProductById($item->getId());
            if (!$product) {
                $continue = false;
            }

            $categories = $product->getCategoryIds();
            $this->_logger->info("-------> ".json_encode($categories));
            /*if (in_array("15", $categories) || in_array("18", $categories) || in_array("135", $categories) || in_array("138", $categories)) {
                $continue = true;
                $this->_logger->info("Entro -> codiicion in_array categori"); 
                if ($item->getQty() != 2 && $item->getQty() != 4) { //neumaticaos
                    $continue = false;
                    $this->_logger->info("Entro codiicion part");
                }

                if ($item->getProductType() == "bundle") {
                    $continue = true;
                    $this->_logger->info("Entro codiicion bundle");
                }
                $inCategory = $continue;
            }*/
            if (in_array("867", $categories) || in_array("18", $categories)) {
                if ($item->getQty()  % 2 == 0) {
                    $continue = true;
                    $this->_logger->info("Entro codiicion part y categoria 15,18");
                }
            }
            if ($item->getProductType() == "bundle") {
                $continue = true;
                $this->_logger->info("Entro codiicion bundle");
            }

            if (in_array("135", $categories) || in_array("138", $categories)) {
                $continue = true;
                $this->_logger->info("Entro codiicion  categoria 135,138");
            }

        }

        return $continue;
    }

}