<?php

namespace Ymina\Balanceo\Model\Total;

use Ymina\Balanceo\Helper\Data;

class Balanceo extends \Magento\Quote\Model\Quote\Address\Total\AbstractTotal
{

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;


    /**
     * @var Data
     */
    protected $helper;

    /**
     * @param Data $helper
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Ymina\Balanceo\Helper\Data $helper,
        \Psr\Log\LoggerInterface $logger
    )
    {
        $this->logger = $logger;
        $this->helper = $helper;
    }

    public function collect(
        \Magento\Quote\Model\Quote                          $quote,
        \Magento\Quote\Api\Data\ShippingAssignmentInterface $shippingAssignment,
        \Magento\Quote\Model\Quote\Address\Total            $total
    )
    {
        parent::collect($quote, $shippingAssignment, $total);
        if (!$this->helper->isValid($quote, $quote->getAllItems())) {
            return $this;
        }

        $balance = $this->helper->getTaxBalanseoAmount();
        $total->setTotalAmount('balanceo', $balance);
        $total->setBaseTotalAmount('balanceo', $balance);

        $total->setBalanseo($balance);
        $total->setBaseBalanseo($balance);
        $total->setGrandTotal($total->getGrandTotal() + $balance);
        $total->setBaseGrandTotal($total->getBaseGrandTotal() + $balance);

        return $this;
    }

    /**
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     */
    protected function clearValues(\Magento\Quote\Model\Quote\Address\Total $total)
    {
        $total->setTotalAmount('subtotal', 0);
        $total->setBaseTotalAmount('subtotal', 0);
        $total->setTotalAmount('tax', 0);
        $total->setBaseTotalAmount('tax', 0);
        $total->setTotalAmount('discount_tax_compensation', 0);
        $total->setBaseTotalAmount('discount_tax_compensation', 0);
        $total->setTotalAmount('shipping_discount_tax_compensation', 0);
        $total->setBaseTotalAmount('shipping_discount_tax_compensation', 0);
        $total->setSubtotalInclTax(0);
        $total->setBaseSubtotalInclTax(0);
    }


    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     * @return array
     */
    public function fetch(\Magento\Quote\Model\Quote $quote, \Magento\Quote\Model\Quote\Address\Total $total)
    {
        if (!$this->helper->isValid($quote, $quote->getItems())) {
            return [];
        }

        return [
            'code' => 'balanceo',
            'title' => __($this->getLabel()),
            'value' => $this->helper->getTaxBalanseoAmount()
        ];
    }


    /**
     * Get Subtotal label
     *
     * @return \Magento\Framework\Phrase
     */
    public function getLabel()
    {
        return $this->helper->getTaxBalanseoTitle();
    }

}