<?php
/**
 * Copyright © agsofware All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\CancelPending\Console\Command;

use Psr\Log\LoggerInterface;
use Ymina\CancelPending\Helper\Data;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\OrderFactory;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Transbank\Webpay\Model\WebpayOrderData;

class CanceleOrder extends Command
{

    const NAME_ARGUMENT = "name";
    const NAME_OPTION = "option";
    private \Magento\Framework\App\State $state;
    private OrderFactory $_orderModel;
    private \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate;
    private Data $data;
    private \Transbank\Webpay\Model\WebpayOrderDataFactory $webpayOrderDataFactory;
    private OrderRepositoryInterface $orderRepository;
    private SearchCriteriaBuilder $searchCriterial;
    private LoggerInterface $_logger;

    public function __construct(
        Data                                                 $data,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Transbank\Webpay\Model\WebpayOrderDataFactory       $webpayOrderDataFactory,
        \Magento\Framework\App\State                         $state,
        OrderRepositoryInterface                             $orderRepository,
        SearchCriteriaBuilder                                $searchCriteriaBuilder,
        OrderFactory $_orderModel
    )
    {
        $this->_orderModel = $_orderModel;
        $this->searchCriterial = $searchCriteriaBuilder;
        $this->orderRepository = $orderRepository;
        $this->localeDate = $localeDate;
        $this->data = $data;
        $this->state = $state;
        $this->webpayOrderDataFactory = $webpayOrderDataFactory;

        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface  $input,
        OutputInterface $output
    )
    {
        $this->state->setAreaCode(\Magento\Framework\App\Area::AREA_FRONTEND); // or \Magento\Framework\App\Area::AREA_ADMINHTML, depending on your needs

        $name = $input->getArgument(self::NAME_ARGUMENT);
        $option = $input->getOption(self::NAME_OPTION);
        $isEnabled = $this->data->getGeneralConfig(Data::ENABLE);
        $from_date = $this->data->getGeneralConfig(Data::FROM_DATE_TEXT);
        $output->writeln("Hora ".$from_date);
        $todayStartOfDayDate = $this->localeDate->date(strtotime($from_date))->setTime(00, 00, 00)->format('Y-m-d H:i:s');

        if ($isEnabled) {

            $collection = $this->_orderModel->create()->getCollection();
            $status = $this->data->getGeneralConfig(Data::PRIMARY_STATUS);
            $statusExp = explode(',', $status);
            if ($statusExp) {

                $collection->addFieldToFilter(
                    'status',
                    ['in' => $statusExp]
                )->addFieldToFilter('created_at',["gt" => $todayStartOfDayDate]);

                $limitCollection = $this->data->getGeneralConfig(Data::BATCH);
                $collection->getSelect()->limit($limitCollection);
                $output->writeln("Filtrado orden que coinciden con la configuracion");
                foreach ($collection as $k => $order) {
                    $fechaEmitida = $order->getUpdatedAt();
                    $minutes = $this->data->getGeneralConfig(Data::MINUTES);
                    $fechaCaducidad = strtotime("+" . $minutes . " Minutes", strtotime($fechaEmitida));
                    $fechaAhora = strtotime(date('Y-m-d H:i:s'));
                    $pago_ok = $this->checkPay($order);
                    //echo $order->getIncrementId().'<br>';
                    if ($fechaAhora > $fechaCaducidad) {
                        try {

                            $order->cancel();
                            $order->setState('canceled')->setStatus(
                                $this->data->getGeneralConfig(Data::SECONDARY_STATUS)
                            );
                            $output->writeln("Se cancelo la orden " . $order->getIncrementId());
                            $order->addStatusToHistory($order->getStatus(), 'Cron: Orden cancelada por inactividad');

                        } catch (\Throwable $th) {
                        }
                    } else {
                        $order->addStatusToHistory($order->getStatus(), 'Cron: Orden no cancelada, Tiene un pago ok Asociado');
                    }
                   $this->orderRepository->save($order);
                }
            }
        }
        $output->writeln("Ordenes Canceladas");
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("ymina_cancelpending:canceleorder");
        $this->setDescription("Cancelacion de ordenes");
        $this->setDefinition([
            new InputArgument(self::NAME_ARGUMENT, InputArgument::OPTIONAL, "Name"),
            new InputOption(self::NAME_OPTION, "-a", InputOption::VALUE_NONE, "Option functionality")
        ]);
        parent::configure();
    }
    protected function checkPay($order)
    {
        $pay = false;
        try {
            $webpayOrderDataModel = $this->webpayOrderDataFactory->create();
            $webpayOrderData = $webpayOrderDataModel->load($order->getIncrementId(), 'order_id');
            $paymentStatus = $webpayOrderData->getPaymentStatus();
            if ($paymentStatus == WebpayOrderData::PAYMENT_STATUS_SUCCESS) {
                $pay = true;
            }
        } catch (\Exception $e) {
            return false;
        }
        return $pay;
    }

}
