<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\Checkout\Controller\Ajax;

use Custom\City\Model\Resource\Zip\Collection;

class Index extends \Magento\Framework\App\Action\Action
{

    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    protected $resultPageFactory;

    /**
     * @var \Magento\Framework\Json\Helper\Data
     */
    protected $jsonHelper;

    /**
     * @var \Custom\City\Model\Resource\City\Collection
     */
    protected $city;


    /**
     * @var \Custom\City\Model\Resource\Zip\Collection
     */
    protected $zip;

    /**
     * Constructor
     *
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Json\Helper\Data $jsonHelper
     */
    public function __construct(
        \Magento\Framework\App\Action\Context       $context,
        \Magento\Framework\View\Result\PageFactory  $resultPageFactory,
        \Magento\Framework\Json\Helper\Data         $jsonHelper,
        \Custom\City\Model\Resource\City\Collection $city,
        \Custom\City\Model\Resource\Zip\Collection  $zip,
        \Psr\Log\LoggerInterface                    $logger
    )
    {
        $this->resultPageFactory = $resultPageFactory;
        $this->jsonHelper = $jsonHelper;
        $this->logger = $logger;
        $this->city = $city;
        $this->zip = $zip;
        parent::__construct($context);
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        try {
            $action = $this->_request->getParam('action');

            if ($action == "province") {

                $options = [];
                $collection = $this->city
                    ->addFilter('state_id', $this->_request->getParam('regionId'))
                    ->addFilter('status', 1)
                    ->setOrder('city', 'ASC');

                foreach ($collection->getItems() as $item) {
                    $options[] = [
                        "label" => $item->getCity(),
                        "id" => $item->getId(),
                        "value" => $item->getCity(),
                        "title" => $item->getCity()
                    ];
                }

                return $this->jsonResponse($options);
            } elseif ($action == "district") {
                $options = [];
                $cityId= $this->getCityId();
                if (!$cityId) {
                    return $this->jsonResponse($options);
                }
                 $collection = $this->zip
                    ->addFilter('city_id', $cityId)
                    ->addFilter('country_id', 'PE')
                    ->setOrder('zip_name', 'ASC');
                foreach ($collection->getItems() as $item) {
                    $options[] = [
                        "label" => $item->getZipName(),
                        "value" => $item->getZipName(),
                        "title" => $item->getZipName(),
                        "id" => $item->getId(),
                    ];
                }
                return $this->jsonResponse($options);
            }


            return $this->jsonResponse('your response');
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            return $this->jsonResponse($e->getMessage());
        } catch (\Exception $e) {
            $this->logger->critical($e);
            return $this->jsonResponse($e->getMessage());
        }
    }

    /**
     * @return mixed
     */
    public function getCityId()
    {
        $city = $this->_request->getParam('city');
        $item = $this->city->addFilter('city', $city)->getFirstItem();
        return $item->getId();
    }

    /**
     * Create json response
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function jsonResponse($response = '')
    {
        return $this->getResponse()->representJson(
            $this->jsonHelper->jsonEncode($response)
        );
    }
}
