<?php

namespace Ymina\Checkout\Helper;

use Magento\Checkout\Model\Session;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\DataObject;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\Data\OrderPaymentInterface;
use Magento\Sales\Model\OrderFactory;


class DataMercadoPagoResponse extends AbstractHelper
{

    /**
     * @var OrderFactory
     */
    protected $_orderFactory;

    /**
     * @var Session
     */
    protected $_checkoutSession;


    public function __construct(
        Context      $context,
        OrderFactory $orderFactory,
        Session      $checkoutSession,
    )
    {
        $this->_orderFactory = $orderFactory;
        $this->_checkoutSession = $checkoutSession;
        parent::__construct($context);
    }


    /**
     * @param $orderIncrementId
     * @return \Magento\Sales\Model\Order|null
     */
    public function getOrder()
    {
        $orderIncrementId = $this->_checkoutSession->getLastRealOrderId();
        return $this->_orderFactory->create()->loadByIncrementId($orderIncrementId);
    }


    /**
     * @return \Magento\Sales\Model\Order
     */
    public function getOrderById($orderIncrementId)
    {
        return $this->_orderFactory->create()->loadByIncrementId($orderIncrementId);
    }

    /**
     * @return false|float|DataObject|OrderPaymentInterface|mixed|null
     */
    public function getPayment()
    {
        return $this->getOrder()->getPayment();
    }


    /**
     * @return array
     */
    public function getInfoPayment()
    {
        /*$order_id = $this->_checkoutSession->getLastRealOrderId();
        return $this->_core->getInfoPaymentByOrder($order_id);*/
        return  [];
    }


    /**
     * @return string
     * @throws LocalizedException
     */
    public function getPaymentMethod()
    {
        return $this->getPayment()->getMethodInstance()->getCode();
    }


    public function formatDataToMessage($payment)
    {
        $paymentResponse = $payment->getAdditionalInformation();
        $paymentData = [];

        if ($this->getPaymentMethod() == "mercadopago_custom") {
            if (isset($paymentResponse['id'])) {
                $paymentData[] = [
                    "text" => __("ID de pago (Mercado Pago)"),
                    "value" => $paymentResponse['id']
                ];
            }

            if (isset($paymentResponse['card']) && isset($paymentResponse['card']['last_four_digits'])) {
                $paymentData[] = [
                    "text" => __("Tarjeta: **** **** ****"),
                    "value" =>  $paymentResponse['card']['last_four_digits']
                ];
            }
            if(isset($paymentResponse['card']) && isset($paymentResponse['card']['expiration_month']) && isset($paymentResponse['card']['expiration_year'])) {
                $paymentData[] = [
                    "text" => __("Fecha de caducidad"),
                    "value" => $paymentResponse['card']['expiration_month'] . "/" . $paymentResponse['card']['expiration_year']
                ];
            }

            if (isset($paymentResponse['payment_method_id'])) {
                $paymentData[] = [
                    "text" => __("Payment Method"),
                    "value" => $paymentResponse['payment_method_id']
                ];
            }

            if (isset($paymentResponse['card']) && isset($paymentResponse['card']['cardholder']) && isset($paymentResponse['card']['cardholder']['name'])) {
                $paymentData[] = [
                    "text" => __("Nombre del titular de la tarjeta"),
                    "value" => $paymentResponse['card']['cardholder']['name']
                ];
            }

            if (isset($paymentResponse['installments'])) {
                $paymentData[] = [
                    "text" => __("Tipo de Cuotas"),
                    "value" => $paymentResponse['installments']
                ];
            }
            if (isset($paymentResponse['statement_descriptor'])) {
                $paymentData[] = [
                    "text" => __("Descriptor de declaración"),
                    "value" => $paymentResponse['statement_descriptor']
                ];
            }
            if (isset($paymentResponse['status'])) {
                $paymentData[] = [
                    "text" => __("Estado de pago"),
                    "value" => $paymentResponse['status']
                ];
            }
            if (isset($paymentResponse['status_detail'])) {
                $paymentData[] = [
                    "text" => __("Detalle del estado de pago"),
                    "value" => $paymentResponse['status_detail']
                ];
            }

        } else {
            $status = $this->_request->getParam("status");

            $transactionResponse = "Transacción Aprobada";
            if ($status && $status !== "approved") {
                $transactionResponse = "Transacción Rechazada";
            }
            $paymentData[] = [
                "text" => __("Respuesta de la Transacción"),
                "value" => $transactionResponse
            ];

            $payment_id = $this->_request->getParam("payment_id");
            if ($payment_id) {
                $paymentData[] = [
                    "text" => __("Identificación de Pago"),
                    "value" => $payment_id
                ];
            }

            $payment_type = $this->_request->getParam("payment_type");
            if ($payment_type) {
                $card = "";
                if ($payment_type == "credit_card") {
                    $card = "Crédito";
                }
                if ($payment_type == "debit_card") {
                    $card = "Débito";
                }

                $paymentData[] = [
                    "text" => __("Tipo de Pago"),
                    "value" => $card
                ];
            }

            $preference_id = $this->_request->getParam("preference_id");
            if ($preference_id) {
                $paymentData[] = [
                    "text" => __("Identificación de preferencia"),
                    "value" => $preference_id
                ];
            }
        }


        $li = "";
        foreach ($paymentData as $item) {
            $li .= "<li>{$item["text"]}: {$item['value']}</li>";
        }

        $message = "<ul>{$li}</ul>";

        return $message;

    }

    public function getInfoPaymentByOrder()
    {
        return json_encode($this->getOrderById(760027)->getPaymentsCollection()->getData());
    }

}