<?php

namespace Ymina\Checkout\Plugin\Magento\Checkout\Block\Checkout;

use Magento\Checkout\Block\Checkout\LayoutProcessor as Checkout;
use Magento\Store\Model\ScopeInterface;

const XML_PATH_SHOW_RECEIPT_TYPE = 'checkout/options/show_receipt_type';

class LayoutProcessor
{

    const XML_PATH_SHOW_RECEIPT_TYPE = 'checkout/options/show_receipt_type';

    private $scopeConfig;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $_logger;

    protected $eavConfig;

    protected $storeManager;

    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Psr\Log\LoggerInterface                           $logger,
        \Magento\Eav\Model\Config                          $eavConfig,
        \Magento\Store\Model\StoreManagerInterface         $storeManager
    )
    {
        $this->scopeConfig = $scopeConfig;
        $this->_logger = $logger;
        $this->eavConfig = $eavConfig;
        $this->storeManager = $storeManager;
    }

    public function afterProcess(
        Checkout $subject,
        array    $jsLayout
    ): array
    {

        $paymentLayout = $jsLayout['components']['checkout']['children']['steps']
        ['children']['billing-step']['children']['payment']['children'];


        if (isset($paymentLayout['afterMethods']['children']['billing-address-form'])) {

            //postcode
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']['postcode']['visible'] = false;

            //firstname
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['firstname']['config']['placeholder'] = __('Nombre*');
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['firstname']['sortOrder'] = 10;

            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['lastname']['config']['placeholder'] = __('Apellido*');
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['lastname']['sortOrder'] = 20;


            $fax = [
                'component' => 'Ymina_Checkout/js/form/element/checkbox-set',
                'config' => [
                    'customScope' => 'billingAddressshared',
                    'template' => 'Ymina_Checkout/form/element/checkbox-set',
                    'elementTmpl' => 'ui/form/element/checkbox-set',
                    'customEntry' => 'billingAddressshared.fax',
                    'additionalClasses' => 'billing-type-document'
                ],
                'dataScope' => 'billingAddressshared.fax',
                'provider' => 'checkoutProvider',
                'multiple' => false,
                'options' => [
                    [
                        'value' => 'RUT',
                        'label' => 'RUT:'
                    ],
                    [
                        'value' => 'Pasaporte/DNI',
                        'label' => 'Pasaporte/DNI:'
                    ]
                ],
                'sortOrder' => 25,
                'validation' => [
                    'required-entry' => true
                ],
                'visible' => true
            ];

            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['fax'] = $fax;


            //vat_id
            $vat_id = [
                'component' => 'Ymina_Checkout/js/form/element/vat_id',
                'config' => [
                    'customScope' => 'billingAddressshared',
                    'template' => 'ui/form/field',
                    'elementTmpl' => 'Ymina_Checkout/form/element/rut',
                    'customEntry' => 'billingAddressshared.vat_id',
                    'placeholder' => __('Número de documento*')
                ],
                'provider' => 'checkoutProvider',
                'dataScope' => 'billingAddressshared.vat_id',
                'sortOrder' => 30,
                'validation' => [
                    'required-entry' => true
                ],
                'visible' => true
            ];
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['vat_id'] = $vat_id;

            //street
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['street']['sortOrder'] = 45;

            //region_id
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['region_id']['sortOrder'] = 80;

            //city
            $city = [
                'component' => 'Ymina_Checkout/js/view/form/element/city',
                'config' => [
                    'customScope' => 'billingAddressshared',
                    'template' => 'ui/form/field',
                    'elementTmpl' => 'ui/form/element/select'
                ],
                'dataScope' => 'billingAddressshared.city',
                'provider' => 'checkoutProvider',
                'sortOrder' => 90,
                'validation' => [
                    'required-entry' => true
                ],
                'options' => [],
                'filterBy' => [
                    'target' => '<![CDATA[${ $.provider }:${ $.parentScope }.region_id]]>',
                    'field' => 'region_id'
                ],
                'customEntry' => null,
                'visible' => true
            ];

            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['city'] = $city;

            $prefix = [
                'component' => 'Ymina_Checkout/js/form/element/prefix',
                'config' => [
                    'customScope' => 'billingAddressshared',
                    'template' => 'ui/form/field',
                    'elementTmpl' => 'ui/form/element/select'
                ],
                'dataScope' => 'billingAddressshared.prefix',
                'provider' => 'checkoutProvider',
                'sortOrder' => 100,
                'validation' => [
                    'required-entry' => true
                ],
                'options' => [],
                'customEntry' => null,
                'visible' => true
            ];
            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['prefix'] = $prefix;

            $telephone = [
                'component' => 'Ymina_Checkout/js/form/element/telephone',
                'config' => [
                    'customScope' => 'billingAddressshared',
                    'template' => 'ui/form/field',
                    'elementTmpl' => 'ui/form/element/input',
                    'placeholder' => __('Teléfono*'),
                    'tooltip' => false
                ],
                'provider' => 'checkoutProvider',
                'dataScope' => 'billingAddressshared.telephone',
                'sortOrder' => 110,
                'validation' => [
                    'required-entry' => true,
                    'min_text_length' => 8,
                    'max_text_length' => 9
                ],

                'filterBy' => null,
                'customEntry' => null,
                'visible' => true
            ];

            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['telephone'] = $telephone;

            $jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']
            ['country_id']['visible'] = false;

            unset($jsLayout['components']['checkout']['children']['steps']
                ['children']['billing-step']['children']['payment']['children']['afterMethods']
                ['children']['billing-address-form']['children']['form-fields']['children']
                ['rut']);

            $this->_logger->info(json_encode($jsLayout['components']['checkout']['children']['steps']
            ['children']['billing-step']['children']['payment']['children']['afterMethods']
            ['children']['billing-address-form']['children']['form-fields']['children']));
        }
        return $jsLayout;
    }
}
