<?php
namespace Ymina\Checkout\ViewModel\Onepage;

use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\App\Config\ScopeConfigInterface;
class Success implements \Magento\Framework\View\Element\Block\ArgumentInterface {

    /**
     * @var \Magento\Checkout\Model\Session
     */
    protected $_checkoutSession;

    /**
     * @var \Magento\Catalog\Block\Product\ImageBuilder
     */
    protected $imageBuilder;

    /**
     * @var \Magento\Framework\Pricing\Helper\Data
     */
    protected $priceHelper;

    /**
     * @var \Magento\Framework\View\Element\BlockFactory
     */
    protected $_blockFactory;


    protected $branchOffice;

    /**
     * @var \Magento\Sales\Api\OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var string
     */
    const IMAGE_ID = 'mini_cart_product_thumbnail';

    /**
     * @var array
     */
    const STOREPICKUP_CODES = [
        'storepickup_storepickup',
        'storepickup_storepickup_express'
    ];

    protected $_registry;

    protected $configurable;

    protected $productRepository;

    /**
     * Constructor
     *
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Catalog\Block\Product\ImageBuilder $imageBuilder
     */
    public function __construct(
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Catalog\Block\Product\ImageBuilder $imageBuilder,
        \Magento\Framework\Pricing\Helper\Data $priceHelper,
        \Magento\Framework\View\Element\BlockFactory $blockFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\SalesRule\Model\Coupon $coupon,
        \Magento\SalesRule\Model\RuleRepository $rule,
        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ProductRepository $productRepository,
        \Magento\Store\Model\StoreManagerInterface $storeFactory,
        \Magento\Framework\App\Config\ScopeConfigInterface $storeManager,
        \Magento\ConfigurableProduct\Model\Product\Type\Configurable $configurable
    ) {
        $this->_checkoutSession = $checkoutSession;
        $this->imageBuilder = $imageBuilder;
        $this->priceHelper = $priceHelper;
        $this->_blockFactory = $blockFactory;
        $this->orderRepository = $orderRepository;
        $this->_registry = $registry;
        $this->_storeManager = $storeManager;
        $this->_storeFactory = $storeFactory;
        $this->_coupon=$coupon;
        $this->_rule=$rule;
        $this->configurable = $configurable;
        $this->productRepository= $productRepository;
    }

    /**
     * Obtain completed checkout order
     *
     * @return \Magento\Sales\Model\Order|null
     */
    public function getOrder() {
        $order = $this->_checkoutSession->getLastRealOrder();

        if ($order->getId()) {
            return $order;
        } else {
            $id = $this->_checkoutSession->getLastOrderId();
            try {
                return $this->orderRepository->get($id);
            } catch (\Exception $err) {
                //return null;
            }
        }

        return $this->_registry->registry('current_order');
    }
    public function getCoupon($code)
    {
       return $this->_coupon->loadByCode($code);
    }

    /*public function getOrder(){
        if ($this->_checkoutSession->getLastRealOrderId()) {
            //echo $this->_checkoutSession->getLastRealOrderId();
            $order = $this->orderRepository->get($this->_checkoutSession->getLastRealOrderId());
            return $order;
        }
        return false;
    }*/


    /**
     * Obtain an image instance sending the product instance
     *
     * @param \Magento\Catalog\Model\Product|null
     * @return \Magento\Catalog\Block\Product\Image
     */
    public function getProductImage($product) {
        if ($product) {
            return $this->imageBuilder->create($product, self::IMAGE_ID, []);
        }
        return null;
    }

    /**
     * Format the given price into Magento's currency
     *
     * @param string|null
     * @return string
     */
    public function formatPrice($price) {
        return $this->priceHelper->currency($price, true, false);
    }

    /**
     * Get branchoffice name
     *
     * @param Magento\Sales\Model\Order|null $order
     * @return object
     */
    public function getBranchoffice($order) {
        /*if ($order && $this->isStorePickup($order)) {
            $bo = $order->getBranchoffice();
            $box = explode(' -- ', $bo);
            $branchOffice = $this->branchOffice->create();
            $branchOffice->addFieldToFilter('ext_code', ['eq' => $box[0]]);

            return $branchOffice->getFirstItem();
        }*/
        return null;
    }

    public function getRule($id)
    {
        return $this->_rule->getById($id);
    }

    /**
     * Get if is storepickup
     *
     * @param Magento\Sales\Model\Order|null $order
     * @return boolean
     */
    public function isStorePickup($order) {
        if ($order) {
            return in_array($order->getShippingMethod(), self::STOREPICKUP_CODES);
        }
        return false;
    }

    /**
     * Get cms block by id
     *
     * @param string $id
     * @return \Magento\Framework\View\Element\BlockFactory
     */
    public function getBlockById($id) {
        return $this->_blockFactory->createBlock('Magento\Cms\Block\Block')->setBlockId($id);
    }

    public function getDataProductOrder(){
        $order = $this->getOrder();
        $data = array();
        foreach($order->getAllVisibleItems() as $item){
            $color = '';
            if(isset($item->getData('product_options')['attributes_info']) && isset($item->getData('product_options')['attributes_info'][0]) && isset($item->getData('product_options')['attributes_info'][0]['value'])){
                $color = $item->getData('product_options')['attributes_info'][0]['value'];
            }
            $data['products'][]=array('nombre'=>$item->getName(),'sku'=>$item->getSku(),'precio'=>round($item->getPrice()),'qty'=>round($item->getQtyOrdered()),'color' => $color, 'product' =>
                $this->getProduct($item->getSku()));
        }
        $data['ship'] = '$'.number_format(round($order->getShippingAmount()), 0, ',', '.');
        return $data;
    }
    public function getGrandTotal() {
        $idOrden = $this->getOrder()->getIncrementId();
        $data = array();
        $data['buyOrder'] = $idOrden;
        $data['total_amount'] = floatval(str_replace(".","",$this->getTotal()));
        return $data;
    }
    public function getTotal()
    {
        $order = $this->getOrder();
        $total = $order->getBaseGrandTotal();

        if (!$total) {
            $total = $order->getBasePrice() + $order->getBaseShippingAmount();
        }

        $total = number_format(floatval($total), 2, ',', '.');

        return $total;
    }

    public function getParentId()
    {
        return $this->_storeManager->getValue('retailrocket/configuration/partner_id', "website",$this->_storeFactory->getWebsite())??null;
    }

    public function getParentIdProduct($chilId)
    {
        return $this->configurable->getParentIdsByChild($chilId);
    }

    public function getProduct($sku)
    {
        return $this->productRepository->get($sku);
    }

}
