define([
    'jquery',
    'rut',
    'jquery/ui',
    'mage/translate',
    'jquery/validate',
    'mage/validation'
], function ($, rut) {
    'use strict';

    $.validator.addMethod(
        'validate-rut',
        function (value) {
            value = $.trim(value);
            // 153926433
            return rut.validateRutV1(value)
        },

        $.mage.__('Ingrese un RUT válido.')
    );

    $.validator.addMethod(
        'telephone-cl',
        function (value) {
            if ($.trim(value) != '') {
                if (value.length < 8 || value.length > 9) {
                    return false;
                }
            }
            let regex = /^[29][0-9]*$/;
            return $.mage.isEmptyNoTrim(value) || regex.test(value);
        },
        $.mage.__('Solo se aceptan d\xedgitos con longitud entre 8 y 9 caracteres y deben iniciar en 2 o 9.')
    );

    $.validator.addMethod(
        'telephone-cl-2',
        function (value) {
            if ($.trim(value) != '') {
                if (value.length < 8 || value.length > 9) {
                    return false;
                }
            }
            let regex = /^4\d{7,8}$/;
            return $.mage.isEmptyNoTrim(value) || regex.test(value);
        },
        $.mage.__('Solo se aceptan d\xedgitos con longitud entre 8 y 9 caracteres y deben iniciar en 4')
    );

    return function (validator) {
        validator.addRule(
            'validate-rut',
            function (value, element) {
                return $.validator.methods['validate-rut'].call(this, value, element);
            },
            $.mage.__('Ingrese un RUT válido.')
        );

        validator.addRule(
            'telephone-cl',
            function (value) {
               return $.validator.methods['telephone-cl'].call(this, value);
            },
            $.mage.__('Solo se aceptan d\xedgitos con longitud entre 8 y 9 caracteres y deben iniciar en 2 o 9.')
        )
        validator.addRule(
            'telephone-cl-2',
            function (value) {
                return $.validator.methods['telephone-cl-2'].call(this, value);
            },
            $.mage.__('Solo se aceptan d\xedgitos con longitud entre 8 y 9 caracteres y deben iniciar en 4')
        )
        return validator;
    }
});