define([
    'jquery',
    'underscore',
    'uiRegistry',
    'Magento_Ui/js/form/element/select',
    'mage/translate',
    'Ymina_Checkout/js/model/shipping-type',
    'Magento_Checkout/js/model/shipping-rate-service',
    'Magento_Checkout/js/model/shipping-rate-processor/new-address',
    'Magento_Checkout/js/model/shipping-service',
    'Magento_Checkout/js/model/quote',
    'Magento_Customer/js/model/customer',
    'Magento_Customer/js/model/address-list',
], function ($, _, registry, Select, $t, shippingType, shippingRateServices, newAddress, shippingServices, quote, customer, addressList) {

    return Select.extend({
        defaults: {
            skipValidation: false,
            imports: {
                update: '${ $.parentName }.city:value',
                branchOffice: '${ $.parentName }.branchOffice'
            },
            options: [{title: $t("Sucursal*"), value: "", label: $t("Sucursal*")}],
            visible: false
        },

        /**
         * {@inheritdoc}
         */
        initialize: function () {
            this._super();
            let self = this;
            shippingType.type.subscribe(function (value) {
                self.visible(value === "pickupInStore");
            })
            return this;
        },

        /**
         * On region update we check for city
         *
         * @param {string} city
         */
        update: function (city) {

            if (typeof city === "undefined") return;

            if (shippingType.type() !== 'pickupInStore') return;

            let self = this;
            let skus = window.checkoutConfig.quoteItemData.map(function (item) {
                return {sku: item.sku, qty: item.qty};
            });

            $.ajax({
                url: '/branchoffices/xpec/index',
                data: {
                    action: 'branchOfficesByComuna',
                    comuna_id: city,
                    skus: skus,
                    address_type: 3
                },
                type: 'post',
                dataType: 'json',
                context: this,
                showLoader: true,
                success: function (response) {
                    if (response.result) {
                        let options = response.result.map(function (item) {
                            return {title: item.name, value: item.value, label: item.name}
                        })
                        self.visible(!!options.length)
                        self.validation['required-entry'] = !!options.length;
                        self.options(options)
                    }
                },
            })
        },
        hasChanged: function () {

            if (this.value()) {
                const sucursal = this.value()
                newAddress.getRates(quote.shippingAddress());
                let shippingAddress = quote.shippingAddress();
                $.ajax({
                    url: '/branchoffices/xpec/index',
                    data: {
                        action: 'branchOffice',
                        id: sucursal,
                        address_type: 3
                    },
                    type: 'post',
                    dataType: 'json',
                    context: this,
                    showLoader: true,
                    success: function (response) {
                        if (response.result) {
                            if (response.result) {
                                const office = response.result[0];
                                $("input[name='street[0]']").val(office.address).trigger("change")
                                $("input[name='street[1]']").val(office.ext_code).trigger("change")
                                $("input[name='street[2]']").val(office.name).trigger("change")

                                if (customer.isLoggedIn() && addressList().length) {
                                    shippingAddress.street[0] = office.address;
                                    shippingAddress.street[1] = office.ext_code;
                                    shippingAddress.street[2] = office.name;
                                    quote.shippingAddress(shippingAddress);
                                }
                            }
                        }
                    },
                })
            }
        }
    })
})
