define([
    'jquery',
    'ko',
    'Magento_Checkout/js/checkout-data',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/model/step-navigator',
    'Magento_Checkout/js/model/payment/additional-validators',
    'Magento_Checkout/js/action/place-order',
    'Magento_Ui/js/model/messages',
    'uiLayout',
    'Magento_Checkout/js/action/get-payment-information'
], function (
    $,
    ko,
    checkoutData,
    quote,
    stepNavigator,
    additionalValidators,
    placeOrderAction,
    Messages,
    layout,

    getPaymentInformation,
    ) {

    let mixin = {

        redirectAfterPlaceOrder: true,
        isPlaceOrderActionAllowed: ko.observable(quote.billingAddress() != null),

        initialize: function () {
            this._super()
            this.initChildren();
            quote.billingAddress.subscribe(function (address) {
                this.isPlaceOrderActionAllowed(address !== null);
            }, this);
            return this;
        },


        /**
         * Initialize child elements
         *
         * @returns {Component} Chainable.
         */
        initChildren: function () {
            this.messageContainer = new Messages();
            this.createMessagesComponent();

            return this;
        },

        /**
         * Create child message renderer component
         *
         * @returns {Component} Chainable.
         */
        createMessagesComponent: function () {

            var messagesComponent = {
                parent: this.name,
                name: this.name + '.messages',
                displayArea: 'messages',
                component: 'Magento_Ui/js/view/messages',
                config: {
                    messageContainer: this.messageContainer
                }
            };

            layout([messagesComponent]);

            return this;
        },


        placeOrder: function (data, event) {
            let self = this;

            if (event) {
                event.preventDefault();
            }

            if (this.validatePayment() &&
                additionalValidators.validate() &&
                this.isPlaceOrderActionAllowed() === true
            ) {
                this.isPlaceOrderActionAllowed(false);
                $('.payment-method._active').find('[type="submit"]').trigger('click');
                return true;
            }

            return false;
        },


        /**
         * Get payment method data
         */
        getData: function () {
            return {
                'method': checkoutData.getSelectedPaymentMethod(),
                'po_number': null,
                'additional_data': null
            };
        },


        isChecked: ko.computed(function () {
            return quote.paymentMethod() ? quote.paymentMethod().method : null;
        }),


        /**
         * @return {Boolean}
         */
        validatePayment: function () {
            return checkoutData.getSelectedPaymentMethod()
        },

        /**
         * Navigate method.
         */
        navigate: function () {
            var self = this;

            if (!self.hasShippingMethod()) {
                this.isVisible(false);
                stepNavigator.setHash('shipping_methods');
            } else {
                getPaymentInformation().done(function () {
                    self.isVisible(true);
                });
            }
        },
    }


    return function (target) {
        return target.extend(mixin)
    }
})