define([
    'ko',
    'uiComponent',
    'underscore',
    'mage/translate',
    'Magento_Checkout/js/model/step-navigator',
    'Magento_Checkout/js/model/shipping-service',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/action/set-shipping-information',
    'Magento_Checkout/js/action/select-shipping-method',
    'Magento_Checkout/js/checkout-data',
], function (
    ko,
    Component,
    _,
    $t,
    stepNavigator,
    shippingService,
    quote,
    setShippingInformationAction,
    selectShippingMethodAction,
    checkoutData
) {
    'use strict';

    return Component.extend({
        defaults: {
            template: 'Ymina_Checkout/shipping-methods',
            shippingMethodListTemplate: 'Magento_Checkout/shipping-address/shipping-method-list',
            shippingMethodItemTemplate: 'Magento_Checkout/shipping-address/shipping-method-item',
        },

        // add here your logic to display step,
        isVisible: ko.observable(false),
        visible: ko.observable(!quote.isVirtual()),
        errorValidationMessage: ko.observable(false),

        /**
         * @returns {*}
         */
        initialize: function () {
            this._super();
            stepNavigator.registerStep(
                'shipping_methods',
                null,
                'Métodos de entrega',
                this.isVisible,
                _.bind(this.navigate, this),
                this.sortOrder
            );
            let self = this;

            stepNavigator.steps.subscribe(function (steps) {
                if (window.location.hash === "#shipping" && window.localStorage.getItem('removeShippingMethod') === "true") {
                    selectShippingMethodAction(null);
                }
            })

            return this;
        },

        /**
         * The navigate() method is responsible for navigation between checkout steps
         * during checkout. You can add custom logic, for example some conditions
         * for switching to your custom step
         * When the user navigates to the custom step via url anchor or back button we_must show step manually here
         */
        navigate: function () {
            let self = this;
            if (!quote.shippingAddress()) {
                this.isVisible(false);
                stepNavigator.setHash('shipping');
            } else {
                self.isVisible(true);
            }
        },

        /**
         * @returns void
         */
        navigateToNextStep: function () {
            stepNavigator.next();
        },

        hideErrorValidationMessage: function (){
            let self = this;
            setTimeout(function () {
                self.errorValidationMessage(false)
            },5000)
        },

        setShippingInformation: function () {
            let self = this;

            quote.shippingMethod.subscribe(function (value) {
                if (value) {
                    self.errorValidationMessage(false)
                }
            })

            if (!quote.shippingMethod()) {
                this.errorValidationMessage(
                    $t('The shipping method is missing. Select the shipping method and try again.')
                );
                return false;
            }

            if (!quote.shippingMethod()['method_code']) {
                this.errorValidationMessage(
                    $t('The shipping method is missing. Select the shipping method and try again.')
                );
            }

            setShippingInformationAction().done(
                function () {
                    stepNavigator.next();
                    window.localStorage.setItem('removeShippingMethod', false)
                }
            )
        },


        back: function () {
            stepNavigator.setHash('shipping');
        },

        /**
         * Shipping Method View
         */
        rates: shippingService.getShippingRates(),
        isLoading: shippingService.isLoading,
        isSelected: ko.computed(function () {
            return quote.shippingMethod() ?
                quote.shippingMethod()['carrier_code'] + '_' + quote.shippingMethod()['method_code'] :
                null;
        }),

        /**
         * @param {Object} shippingMethod
         * @return {Boolean}
         */
        selectShippingMethod: function (shippingMethod) {
            selectShippingMethodAction(shippingMethod);
            checkoutData.setSelectedShippingRate(shippingMethod['carrier_code'] + '_' + shippingMethod['method_code']);

            return true;
        },
    });
});
