define([
    'jquery',
    'underscore',
    'ko',
    'mage/translate',
    'Magento_Checkout/js/model/quote',
    'Magento_Checkout/js/model/step-navigator',
    'Magento_Customer/js/model/address-list',
    'Ymina_Checkout/js/helpers',
    'Ymina_Checkout/js/model/shipping-type',
    'Magento_Checkout/js/model/shipping-rate-processor/new-address',
    'Magento_Customer/js/model/customer',
    'uiRegistry',
    'Magento_Checkout/js/model/address-converter',
    'Magento_Checkout/js/action/select-shipping-address',
    'Magento_Checkout/js/action/select-billing-address',
    'Magento_Checkout/js/model/checkout-data-resolver',
    'Magento_Checkout/js/checkout-data',
    'Magento_Checkout/js/action/set-shipping-information'
], function (
    $,
    _,
    ko,
    $t,
    quote,
    stepNavigator,
    addressList,
    helpers,
    shippingType,
    newAddress,
    customer,
    registry,
    addressConverter,
    selectShippingAddress,
    selectBillingAddress,
    checkoutDataResolver,
    checkoutData,
    setShippingInformationAction

) {
    'use strict'

    var self;
    let mixin = {

        shippingAction: shippingType.type,
        home: ko.observable($t('Envío a domicilio')),
        pickup: ko.observable($t('Retiro en tienda')),
        visibleShowMore: ko.observable(addressList().length > helpers.limitAddress()),
        errorValidationMessage: ko.observable(false),
        shippingFormTemplate: 'Magento_Checkout/shipping-address/form',

        initialize: function () {
            this._super();

            if (window.checkoutConfig.selectedShippingMethod
                && window.checkoutConfig.selectedShippingMethod.carrier_code === "amstorepickup"
            ) {
                window.checkoutConfig.selectedShippingMethod.carrier_code = null;
                //quote.shippingMethod(null);
                //pickup.isPickup(false)
            }

            if (!quote.isVirtual()) {
                let steps = stepNavigator.steps().map(step => {
                    if (step.code === "shipping") {
                        step.title = $t('Personal information')
                    }
                    return step
                })
                stepNavigator.steps(steps)

            }

            this._query();
            self = this;

            helpers.limitAddress.subscribe(function (limit) {
                if (limit === addressList().length) {
                    self.visibleShowMore(false)
                }
            })


            this.sendAjaxShippingAction(this.shippingAction());
            this.shippingAction.subscribe(function (value) {
                //send ajax
                self.sendAjaxShippingAction(value)
            })
            return this;
        },

        /**
         * Navigator change hash handler.
         *
         * @param {Object} step - navigation step
         */
        navigate: function (step) {
            step && step.isVisible(true);
        },

        _query: function () {
            let mediaQuery = window.matchMedia('(max-width: 768px)');
            if (mediaQuery.matches) {
                this.home($t('Envío'));
                this.pickup($t('Retiro'));
            }
        },

        showMoreAddress: function () {
            helpers.limitAddress(addressList().length)
        },

        sendAjaxShippingAction: function (method) {
            let url = BASE_URL + 'actionShipping/Index/typeDelivery';
            $.ajax({
                url,
                data: {
                    method
                },
                beforeSend: function () {
                    $('body').trigger('processStart');
                },

                success: function (data) {
                    //console.log(data)
                },

                complete: function () {
                    $('body').trigger('processStop');
                }
            })
        },

        /**
         * Set shipping information handler
         */
        setShippingInformation: function () {
            if (this.validateShippingInformation()) {
                quote.billingAddress(null);
                checkoutDataResolver.resolveBillingAddress();
                registry.async('checkoutProvider')(function (checkoutProvider) {
                    var shippingAddressData = checkoutData.getShippingAddressFromData();

                    if (shippingAddressData) {
                        checkoutProvider.set(
                            'shippingAddress',
                            $.extend(true, {}, checkoutProvider.get('shippingAddress'), shippingAddressData)
                        );
                    }
                });
                window.localStorage.setItem('removeShippingMethod', true)
                stepNavigator.next();
            }
        },

        /**
         * Validate guest email
         */
        validateGuestEmail: function () {
            var loginFormSelector = 'form[data-role=email-with-possible-login]';
            $(loginFormSelector).validation();
            return $(loginFormSelector + ' input[type=email]').valid();
        },

        /**
         * @return {Boolean}
         */
        validateShippingInformation: function () {
            var result = this._super(),
                shippingMethod = quote.shippingMethod();

            if (!customer.isLoggedIn() && !this.validateGuestEmail()) {
                return result;
            }

            if (window.checkoutConfig.selectedShippingMethod
                && window.checkoutConfig.selectedShippingMethod['carrier_code'] === "amstorepickup") {
                this.source.set('params.invalid', false);
                this.source.trigger('amStorepickup.data.validate');
                return true;
            }

            if (shippingMethod && shippingMethod['carrier_code'] === 'amstorepickup') {
                this.source.set('params.invalid', false);
                this.source.trigger('amStorepickup.data.validate');
                //return !this.source.get('params.invalid');
                return true;
            }


            var shippingAddress,
                addressData,
                loginFormSelector = 'form[data-role=email-with-possible-login]',
                emailValidationResult = customer.isLoggedIn(),
                field,
                option = _.isObject(this.countryOptions) && this.countryOptions[quote.shippingAddress().countryId],
                messageContainer = registry.get('checkout.errors').messageContainer;
            quote.shippingMethod(null)


            if (!customer.isLoggedIn()) {
                $(loginFormSelector).validation();
                emailValidationResult = Boolean($(loginFormSelector + ' input[name=username]').valid());
            }

            if (this.isFormInline) {
                this.source.set('params.invalid', false);
                this.triggerShippingDataValidateEvent();

                if (emailValidationResult &&
                    this.source.get('params.invalid')
                ) {
                    this.focusInvalid();

                    return false;
                }

                shippingAddress = quote.shippingAddress();
                addressData = addressConverter.formAddressDataToQuoteAddress(
                    this.source.get('shippingAddress')
                );

                //Copy form data to quote shipping address object
                for (field in addressData) {
                    if (addressData.hasOwnProperty(field) &&  //eslint-disable-line max-depth
                        shippingAddress.hasOwnProperty(field) &&
                        typeof addressData[field] != 'function' &&
                        _.isEqual(shippingAddress[field], addressData[field])
                    ) {
                        shippingAddress[field] = addressData[field];
                    } else if (typeof addressData[field] != 'function' &&
                        !_.isEqual(shippingAddress[field], addressData[field])) {
                        shippingAddress = addressData;

                        break;
                    }
                }

                shippingAddress.telephone = shippingAddress.prefix + shippingAddress.telephone;

                if (customer.isLoggedIn()) {
                    shippingAddress['save_in_address_book'] = 1;
                }

                selectShippingAddress(shippingAddress);
                selectBillingAddress(shippingAddress);
                checkoutData.setSelectedBillingAddress(quote.shippingAddress());
                window.localStorage.setItem("address_data", JSON.stringify(quote.shippingAddress()))

            } else if (customer.isLoggedIn() &&
                option &&
                option['is_region_required'] &&
                !quote.shippingAddress().region
            ) {
                messageContainer.addErrorMessage({
                    message: $t('Please specify a regionId in shipping address.')
                });

                return false;
            }

            if (!emailValidationResult) {
                $(loginFormSelector + ' input[name=username]').trigger('focus');

                return false;
            }

            return true;
        },

        isAmGiftcard: function () {
            let quote = checkoutConfig.quoteItemData;
            let result = false;
            quote.forEach(element => {
                if (element.product_type !== "amgiftcard") {
                    return false
                } else {

                    result = true
                }
            })

            return result;
        },
    };

    return function (target) {
        return target.extend(mixin);
    };
})
