<?php

namespace Ymina\CustomSearch\Model\Indexer;

use Magento\Framework\App\ResourceConnection;

class Attributes implements
    \Magento\Framework\Indexer\ActionInterface,
    \Magento\Framework\Mview\ActionInterface
{

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;
    /**
     * @var \Magento\Framework\App\ResourceConnection|mixed
     */
    private ResourceConnection $_resource;


    public function __construct(\Psr\Log\LoggerInterface $logger, ResourceConnection $resourceConnection)
    {
        $this->_logger = $logger;
        $this->_resource = $resourceConnection;
    }

    /*
     * Used by mview, allows process indexer in the "Update on schedule" mode
     */
    public function execute($ids)
    {
        //code here!
        $this->executeFull();
    }

    /*
     *
     * Will take all of the data and reindex
     * Will run when reindex via command line
     */
    public function executeFull()
    {
        try {
            //code here!
            $table = 'ymina_custom_search_index';
            $db = $this->_resource->getConnection('core_write');
            $table = $this->_resource->getTableName($table);
            $db->query("DROP TABLE IF EXISTS $table");
            $db->query("
            CREATE TABLE $table AS
SELECT
    cpe.row_id,
    ancho_neumatico.value AS ancho_neumatico,
    perfil_neumatico.value AS perfil_neumatico,
    aro.value AS aro,
    CASE
        WHEN apernadura.value IS NOT NULL THEN
            SUBSTRING_INDEX(SUBSTRING_INDEX(apernadura.value, ',', numbers.n), ',', -1)
        ELSE
            NULL
    END as apernadura_configurable,
    ancho_offset.value as ancho_offset,
    tipo_de_vehiculo.value AS tipo_de_vehiculo,
    status.value as product_status,
    stock.quantity as product_stock,
    stock.status as product_is_in_stock
FROM
    catalog_product_entity AS cpe
CROSS JOIN
    (SELECT 1 n UNION ALL SELECT 2 UNION ALL SELECT 3 UNION ALL SELECT 4) numbers
LEFT JOIN
    catalog_product_entity_int ancho_neumatico ON cpe.row_id = ancho_neumatico.row_id
    AND ancho_neumatico.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'ancho_neumatico' LIMIT 1)
LEFT JOIN
    catalog_product_entity_int perfil_neumatico ON cpe.row_id = perfil_neumatico.row_id
    AND perfil_neumatico.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'perfil_neumatico' LIMIT 1)
LEFT JOIN
    catalog_product_entity_int aro ON cpe.row_id = aro.row_id
    AND aro.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'aro' LIMIT 1)
LEFT JOIN
    catalog_product_entity_text apernadura ON cpe.row_id = apernadura.row_id
    AND apernadura.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'apernadura_configurable' LIMIT 1)
LEFT JOIN
    catalog_product_entity_int ancho_offset ON cpe.row_id = ancho_offset.row_id
    AND ancho_offset.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'ancho_offset' LIMIT 1)
LEFT JOIN
    catalog_product_entity_int tipo_de_vehiculo ON cpe.row_id = tipo_de_vehiculo.row_id
    AND tipo_de_vehiculo.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'tipo_de_vehiculo' LIMIT 1)
LEFT JOIN
    catalog_product_entity_int visibility ON cpe.row_id = visibility.row_id
    AND visibility.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'visibility' LIMIT 1)
INNER JOIN
    catalog_product_entity_int status ON cpe.row_id = status.row_id
    AND status.attribute_id = (SELECT attribute_id FROM eav_attribute WHERE attribute_code = 'status' LIMIT 1)
INNER JOIN
    inventory_source_item stock ON cpe.sku = stock.sku
WHERE
    status.value = 1 AND stock.quantity >= 0 AND visibility.value=4
    AND (apernadura.value IS NULL OR numbers.n <= LENGTH(apernadura.value) - LENGTH(REPLACE(apernadura.value, ',', '')) + 1);

            ");
            return true;
        } catch (\Exception $exception) {
            $this->_logger->critical($exception->getMessage());
        }
        return true;

    }


    /*
     * Works with a set of entity changed (may be massaction)
     */
    public function executeList(array $ids)
    {
        //code here!
        $this->executeFull();
    }


    /*
     * Works in runtime for a single entity using plugins
     */
    public function executeRow($id)
    {
        //code here!
        $this->executeFull();
    }
}
