define(["jquery", "ko", "uiComponent", "mage/url"], function ($, ko, Component, $url) {
    return Component.extend({
        defaults: {
            template: "Ymina_CustomSearch/search",
            templateField: "Ymina_CustomSearch/form/inputs",
            switchState: ko.observable(1),
            neomaticosOptions: ko.observable([]),
            isSelecd: ko.observable(false),
            observableOptions: {
                "tipo_de_vehiculo": ko.observable([]),
                "ancho_neumatico": ko.observable([]),
                "perfil_neumatico": ko.observable([]),
                "aro": ko.observable([])
            },
            observableOptionsL: {
                "aro": ko.observable([]),
                "apernadura_configurable": ko.observable([]),
                "ancho_offset": ko.observable([])
            }

        },

        initialize: function () {
            this._super();
            var self = this;
            this.options = JSON.parse(this.options);
            console.log(this.options)
            this.options.neumaticos.forEach(function (option) {
                if (self.observableOptions[option.code]) {
                    self.observableOptions[option.code](option);
                }
            })
            this.options.llantas.forEach(function (option) {
                if (self.observableOptionsL[option.code]) {
                    if(option.code==="aro"){
                        option.options=option.options.filter(item => !item.label.includes('.'));
                    }
                    self.observableOptionsL[option.code](option);
                }
            })
            $(document).on('change', 'select.filter-search-custom', function () {
                if (this.value) {
                    self.isSelecd(true)
                    self.processData(self.collectData());
                }
            });
            return this;
        },

        switchOption: function (data, event) {
            const value = parseInt($(event.target).data("switch"));
            const $containerTab = $('.filter-container .filter');
            $containerTab.removeClass(value === 1 ? 'llantas' : 'neumaticos').addClass(value === 1 ? 'neumaticos' : 'llantas');
            $('html').addClass('custom-search-active')
            $('.sections-overlay').show();
            $('.filter-container').addClass('active');
            this.switchState(value)
        },

        processData: function (queryParams) {
            this.collectAndSendData(queryParams);
        },

        collectData: function () {
            const $selects = $("select.filter-search-custom");
            let queryParams = {};
            $selects.each(function () {
                let value = $(this).val();
                if (value !== "") {
                    let key = $(this).attr("id");
                    queryParams[key] = value;
                }
            });
            return queryParams;
        },

        searchBuilder: function () {
            let queryParams = this.collectData()
            let category = this.switchState() !== 1 ? "llantas.html" : "neumaticos.html";
            let queryString = $.param(queryParams);
            let url = $url.build(`${category}?${queryString}`);

            // Redirect to the built url
            window.location.href = url;
        },

        collectAndSendData: function (queryParams) {
            let self = this;
            $.ajax({
                url: 'customsearch/attributes/filter',
                type: 'GET',
                data: queryParams,
                success: function (response) {
                    self.filterOptions(response)
                },
                error: function (error) {
                    // Handle error
                    console.log(error);
                }
            });
        },

        sendData: function (queryParams, buildCb, sendCb) {
            let category = this.switchState() !== 1 ? "llantas.html" : "neumaticos.html";
            let queryString = $.param(queryParams);
            let url = $url.build(`${category}?${queryString}`);

            if (buildCb && typeof buildCb === "function") {
                buildCb(queryParams);
            }

            if (sendCb && typeof sendCb === "function") {
                sendCb(queryParams);
            }
        },

        filterOptions: function (ajaxResponseArray) {
            let self = this;
            let _optionsA = this.options.neumaticos;
            if (self.switchState() === 2) {
                _optionsA = this.options.llantas;
            }
            let filteredNeomaticosOptions = _optionsA.map(function (option) {
                let newOption = {...option};
                newOption.options = option.options.filter(function (opt) {
                    return ajaxResponseArray.some(function (responseItem) {
                        return responseItem.hasOwnProperty(option.code) && responseItem[option.code] === opt.value;
                    });
                });
                return newOption;
            });

            // Filter to keep only those with non-empty options
            filteredNeomaticosOptions = filteredNeomaticosOptions.filter(function (option) {
                return option.options.length > 0;
            });
            filteredNeomaticosOptions.forEach(filter => {
                if (self.switchState() === 2) {
                    self.observableOptionsL[filter.code](filter)
                } else {
                    self.observableOptions[filter.code](filter)
                }


            })
        },
        clearFilters: function () {
            let self = this;
            self.isSelecd(false)
            this.options.neumaticos.forEach(function (option) {
                if (self.observableOptions[option.code]) {
                    self.observableOptions[option.code](option);
                }
            })
            this.options.llantas.forEach(function (option) {
                if (self.observableOptionsL[option.code]) {
                    if(option.code==="aro"){
                        option.options=option.options.filter(item => !item.label.includes('.'));
                    }
                    self.observableOptionsL[option.code](option);
                }
            })
            const $selects = $("select.filter-search-custom");
            let queryParams = {};
            $selects.each(function () {
                let value = $(this).val("");
            });
        }
    });
});
