<?php

namespace Ymina\CustomerField\Block\Customer\Widget;

use Magento\Customer\Api\AddressMetadataInterface;
use Magento\Customer\Model\AddressFactory;
use Magento\Customer\Model\Address;
use Magento\Eav\Model\AttributeRepository;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;


class Rut extends Template
{
    protected AddressFactory $addressFactory;
    private AttributeRepository $attributeRepository;
    private ?Address $address = null;

    public function __construct(
        AddressFactory $addressFactory,
        AttributeRepository $attributeRepository,
        Context $context,

    ) {
        $this->addressFactory = $addressFactory;
        $this->attributeRepository = $attributeRepository;
        $this->setTemplate('Ymina_CustomerField::address/widget/rut.phtml');
        parent::__construct($context);
    }

    private function getAddress(): Address
    {
        if (null === $this->address) {
            $addressId = $this->getRequest()->getParam('id');
            $this->address = $this->addressFactory->create()->load($addressId);
        }
        return $this->address;
    }

    public function getValue(): ?string
    {
        return $this->getAttributeValue('rut');
    }

    public function getTipoDocumentoValue(): ?string
    {
        return $this->getAttributeValue('tipo_documento');
    }

    private function getAttributeValue(string $attributeCode): ?string
    {
        $value = $this->getAddress()->getData($attributeCode);
        if (!$value) {
            $value = $this->getRequest()->getParam($attributeCode);
        }
        return $value;
    }

    public function getAttributeByCode($code): \Magento\Eav\Api\Data\AttributeInterface
    {
        return $this->attributeRepository->get(AddressMetadataInterface::ENTITY_TYPE_ADDRESS, $code);
    }
}
