<?php
/**
 * Copyright © Magento, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Ymina\ECOTax\Block\Adminhtml\Sales;


use Magento\Sales\Model\Order;

class Totals extends \Magento\Framework\View\Element\Template
{
    /**
     * @var Order
     */
    protected $_order;
    /**
     * @var \Magento\Framework\DataObject
     */
    protected $_source;

    private $helper;

    private $quoteFactory;

    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Ymina\ECOTax\Helper\Data                        $helper,
        \Magento\Quote\Model\QuoteFactory                $quoteFactory,
        array                                            $data = []
    )
    {
        $this->helper = $helper;
        $this->quoteFactory = $quoteFactory;
        parent::__construct($context, $data);
    }

    public function getSource()
    {
        return $this->_source;
    }

    public function displayFullSummary()
    {
        return true;
    }

    public function initTotals()
    {
        $parent = $this->getParentBlock();
        $this->_order = $parent->getOrder();
        $this->_source = $parent->getSource();

        $store = $this->getStore();
        if (!$this->helper->isEnabledTaxEco()) {
            return $this;
        }

        $quote = $this->quoteFactory->create()->load($this->_order->getQuoteId());

        if (!$this->helper->getProductInCategory($quote, $this->_order->getAllItems())) {
            return $this;
        }

        $customAmount = new \Magento\Framework\DataObject(
            [
                'code' => 'taxEco',
                'label' => __($this->helper->getTaxEcoTitle()),
                'strong' => false,
                'value' => $this->helper->getTaxEcoAmount(),
                'base_value' => $this->helper->getTaxEcoAmount()
            ]
        );
        $parent->addTotal($customAmount, 'taxEco');
        return $this;
    }

    /**
     * Get order store object
     *
     * @return \Magento\Store\Model\Store
     */
    public function getStore()
    {
        return $this->_order->getStore();
    }

    /**
     * @return Order
     */
    public function getOrder()
    {
        return $this->_order;
    }

    /**
     * @return array
     */
    public function getLabelProperties()
    {
        return $this->getParentBlock()->getLabelProperties();
    }

    /**
     * @return array
     */
    public function getValueProperties()
    {
        return $this->getParentBlock()->getValueProperties();
    }
}
