<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\ECOTax\Helper;

use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Quote\Model\Quote;
use Magento\Store\Model\ScopeInterface;

class Data extends AbstractHelper
{
    const XML_PATH_SHOW_TAX_ECO = 'checkout/options/enable_tax_eco';
    const XML_PATH_TITLE_TAX_ECO = 'checkout/options/title_tax_eco';
    const XML_PATH_AMOUNT_TAX_ECO = 'checkout/options/amount_tax';

    /**
     * @var ProductRepository
     */
    private $_productRepository;

    /**
     * @param \Magento\Framework\App\Helper\Context $context
     * @param ProductRepository $productRepository
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        ProductRepository                     $productRepository,

    )
    {
        $this->_productRepository = $productRepository;
        parent::__construct($context);
    }

    /**
     * @return int
     */
    public function isEnabledTaxEco()
    {
        return (int)$this->scopeConfig->getValue(self::XML_PATH_SHOW_TAX_ECO, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @return mixed
     */
    public function getTaxEcoTitle()
    {
        return $this->scopeConfig->getValue(self::XML_PATH_TITLE_TAX_ECO, ScopeInterface::SCOPE_STORE);
    }


    /**
     * @return float
     */
    public function getTaxEcoAmount()
    {
        return (float)$this->scopeConfig->getValue(self::XML_PATH_AMOUNT_TAX_ECO, ScopeInterface::SCOPE_STORE);
    }

    /**
     * @param $sku
     * @return \Magento\Catalog\Api\Data\ProductInterface|\Magento\Catalog\Model\Product|null
     */
    public function getProductBySku($sku)
    {
        try {
            return $this->_productRepository->get($sku);
        } catch (\Exception) {
            return null;
        }

    }

    /**
     * @param Quote $quote
     * @param $items
     * @return bool
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getProductInCategory($quote, $items = null)
    {
        if (!$items) {
            $items = $quote->getItems();
        }

        if (!$items) {
            return false;
        }


        foreach ($items as $item) {
            $product = $this->getProductBySku($item->getSku());
            if (!$product) {
                return false;
            }
            if (in_array('3', $this->getProductBySku($item->getSku())->getCategoryIds())) {
                return true;
            }
        }
        return false;
    }
}