<?php

namespace Ymina\ECOTax\Model\Total;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Quote\Model\Quote\Address\Total\AbstractTotal;
use Magento\Quote\Model\Quote;
use Magento\Quote\Api\Data\ShippingAssignmentInterface;
use Magento\Quote\Model\Quote\Address\Total;
use Magento\Setup\Exception;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\Pricing\PriceCurrencyInterface as PriceRounder;
use Magento\Catalog\Model\ProductRepository;
use Ymina\ECOTax\Helper\Data;


class ECOTax extends AbstractTotal
{

    /**
     * @var PriceRounder
     */
    private $priceRounder;


    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var LoggerInterface
     */
    private $logger;

    protected $_productRepository;

    private $data;

    public function __construct(
        ?PriceRounder        $priceRounder,
        ScopeConfigInterface $scopeConfig,
        Data $data,
        LoggerInterface      $logger,
    )
    {
        $this->priceRounder = $priceRounder ?: ObjectManager::getInstance()->get(PriceRounder::class);
        $this->scopeConfig = $scopeConfig;
        $this->data = $data;
        $this->logger = $logger;
        $this->setCode('taxEco');
    }

    /**
     *
     * @param Quote $quote
     * @param ShippingAssignmentInterface $shippingAssignment
     * @param Total $total
     * @return $this
     */
    public function collect(
        Quote                       $quote,
        ShippingAssignmentInterface $shippingAssignment,
        Total                       $total
    )
    {
        parent::collect($quote, $shippingAssignment, $total);

        if (!$this->data->isEnabledTaxEco()) {
            return $this;
        }

        $items = $shippingAssignment->getItems();
        if (!count($items)) {
            return $this;
        }

        if (!$this->data->getProductInCategory($quote)) {
            return $this;
        }

        $amount = $this->data->getTaxEcoAmount();

        $total->setTotalAmount($this->getCode(), $amount);
        $total->setBaseTotalAmount($this->getCode(), $amount);
        $total->setTaxEco($amount);
        $total->setBaseTaxEco($amount);
        $total->setGrandTotal($total->getGrandTotal() + $amount);
        $total->setBaseGrandTotal($total->getBaseGrandTotal() + $amount);

        return $this;
    }

    /**
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     */
    protected function clearValues(\Magento\Quote\Model\Quote\Address\Total $total)
    {
        $total->setTotalAmount('subtotal', 0);
        $total->setBaseTotalAmount('subtotal', 0);
        $total->setTotalAmount('tax', 0);
        $total->setBaseTotalAmount('tax', 0);
        $total->setTotalAmount('discount_tax_compensation', 0);
        $total->setBaseTotalAmount('discount_tax_compensation', 0);
        $total->setTotalAmount('shipping_discount_tax_compensation', 0);
        $total->setBaseTotalAmount('shipping_discount_tax_compensation', 0);
        $total->setSubtotalInclTax(0);
        $total->setBaseSubtotalInclTax(0);
    }

    /**
     * @param \Magento\Quote\Model\Quote $quote
     * @param \Magento\Quote\Model\Quote\Address\Total $total
     * @return array
     */
    public function fetch(
        \Magento\Quote\Model\Quote               $quote,
        \Magento\Quote\Model\Quote\Address\Total $total
    )
    {
        if (!$this->data->isEnabledTaxEco()) {
            return [];
        }

        try {
            if (!$this->data->getProductInCategory($quote)) {
                return [];
            }

        } catch (Exception $exception) {
            $this->logger->critical($exception->getMessage());
            return [];
        }


        return [
            'code' => $this->getCode(),
            'title' => __($this->data->getTaxEcoTitle()),
            'value' => $this->data->getTaxEcoAmount()
        ];
    }


    /**
     * @return \Magento\Framework\Phrase
     */
    public function getLabel()
    {
        return $this->data->getTaxEcoTitle();
    }
}
