<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\HomeDelivery\Api;

use Magento\Framework\Api\SearchCriteriaInterface;

interface FeeRepositoryInterface
{

    /**
     * Save Fee
     * @param \Ymina\HomeDelivery\Api\Data\FeeInterface $fee
     * @return \Ymina\HomeDelivery\Api\Data\FeeInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function save(
        \Ymina\HomeDelivery\Api\Data\FeeInterface $fee
    );

    /**
     * Retrieve Fee
     * @param string $feeId
     * @return \Ymina\HomeDelivery\Api\Data\FeeInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function get($feeId);

    /**
     * Retrieve Fee matching the specified criteria.
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
     * @return \Ymina\HomeDelivery\Api\Data\FeeSearchResultsInterface
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $searchCriteria
    );

    /**
     * Delete Fee
     * @param \Ymina\HomeDelivery\Api\Data\FeeInterface $fee
     * @return bool true on success
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function delete(
        \Ymina\HomeDelivery\Api\Data\FeeInterface $fee
    );

    /**
     * Delete Fee by ID
     * @param string $feeId
     * @return bool true on success
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function deleteById($feeId);
}

