<?php

namespace Ymina\StructuredData\Block;

use Magento\Catalog\Model\Category;

class CategoryScript extends \Magento\Catalog\Block\Category\View
{


    /**
     * @var \Magento\Framework\Json\Helper\Data
     */
    protected $serialized;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Theme\Block\Html\Header\Logo
     */
    protected $logo;

    /**
     * @var \Magento\Catalog\Model\CategoryFactory
     */
    protected $categoryFactory;

    protected $indexScript;


    /**
     * @param \Magento\Framework\View\Element\Template\Context $context
     * @param \Magento\Catalog\Model\Layer\Resolver $layerResolver
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Helper\Category $categoryHelper
     * @param \Magento\Framework\Json\Helper\Data $serialized
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Theme\Block\Html\Header\Logo $logo
     * @param \Magento\Catalog\Model\CategoryFactory $categoryFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Catalog\Model\Layer\Resolver            $layerResolver,
        \Magento\Framework\Registry                      $registry,
        \Magento\Catalog\Helper\Category                 $categoryHelper,
        \Magento\Framework\Json\Helper\Data              $serialized,
        \Magento\Store\Model\StoreManagerInterface       $storeManager,
        \Magento\Theme\Block\Html\Header\Logo            $logo,
        \Magento\Catalog\Model\CategoryFactory           $categoryFactory,
        \Ymina\StructuredData\Block\IndexScript          $indexScript,
        array                                            $data = [])
    {
        $this->serialized = $serialized;
        $this->_storeManager = $storeManager;
        $this->logo = $logo;
        $this->categoryFactory = $categoryFactory;
        $this->indexScript = $indexScript;
        parent::__construct($context, $layerResolver, $registry, $categoryHelper, $data);
    }


    /**
     * @return array
     */
    public function getDataToArray()
    {
        $category = $this->getCurrentCategory();


        return [
            "structured" => [
                "@context" => "https://schema.org",
                "@type" => "Brand",
                "@id" => "Brand",
                "additionalType" => $category->getUrl(),
                "alternateName" => $this->getCurrentCategory()->getName(),
                "description" => $category->getMetaDescription(), //$this->getDescription($category),
                "identifier" => $category->getUrl(),
                "image" => $category->getImageUrl(),
                "logo" => $this->logo->getLogoSrc(),
                "mainEntityOfPage" => "https://www.llantasdelpacifico.cl/",
                "name" => $this->getStoreName(),
                "sameAs" => $category->getParentCategory() ? $category->getParentCategory()->getUrl() : "",
            ],
            "removesc" => false,
            "jsonStringify" => true
        ];
    }


    /**
     * @param array $formData
     * @return string
     */
    public function serializedFormData($formData)
    {
        return $this->serialized->jsonEncode($formData);
    }

    /**
     * @return array
     */
    private function getChildCategories()
    {
        $cotegory = $this->getCurrentCategory();
        $categories = [];
        $categories[] = [
            "@type" => "ListItem",
            "position" => 1,
            "item" => [
                "@id" => $cotegory->getUrl(),
                "name" => $cotegory->getName()
            ]
        ];


        foreach ($cotegory->getChildrenCategories() as $subcategorie) {

            if (!$subcategorie->getIsActive()) {
                continue;
            }

            $categories[] = [
                "@type" => "ListItem",
                "position" => 2,
                "item" => [
                    "@id" => $subcategorie->getUrl(),
                    "name" => $subcategorie->getName()
                ]
            ];
        }

        return $categories;

    }

    public function getStoreName()
    {
        return $this->_storeManager->getStore()->getName();
    }

    /**
     * @param Category $currentCategory
     * @return string
     */
    public function getDescription($currentCategory)
    {
        $categoryId = $currentCategory->getId();
        $category = $this->categoryFactory->create()->load($categoryId);
        if ($category) {
            return $category->getMetaDescription();
        }

        return "";
    }

    public function isCategorySucursales() {
        //No existe la categoria sucucursales
        if ($this->getCurrentCategory()->getId() == "") {
            return;
        }
    }

}
