<?php

namespace Ymina\StructuredData\Block;

class ProductScript extends \Magento\Catalog\Block\Product\View
{

    /**
     * @var \Magento\Catalog\Helper\Image
     */
    private $imageHelper;


    /**
     * @var \Magento\Framework\Json\Helper\Data
     */
    protected $serialized;

    protected $logger;
    /**
     * @param Context $context
     * @param \Magento\Framework\Url\EncoderInterface $urlEncoder
     * @param \Magento\Framework\Json\EncoderInterface $jsonEncoder
     * @param \Magento\Framework\Stdlib\StringUtils $string
     * @param \Magento\Catalog\Helper\Product $productHelper
     * @param \Magento\Catalog\Model\ProductTypes\ConfigInterface $productTypeConfig
     * @param \Magento\Framework\Locale\FormatInterface $localeFormat
     * @param \Magento\Customer\Model\Session $customerSession
     * @param ProductRepositoryInterface $productRepository
     * @param \Magento\Framework\Pricing\PriceCurrencyInterface $priceCurrency
     * @param \Magento\Catalog\Helper\Image $image
     * @param array $data
     */
    public function __construct(\Magento\Catalog\Block\Product\Context              $context,
                                \Magento\Framework\Url\EncoderInterface             $urlEncoder,
                                \Magento\Framework\Json\EncoderInterface            $jsonEncoder,
                                \Magento\Framework\Stdlib\StringUtils               $string,
                                \Magento\Catalog\Helper\Product                     $productHelper,
                                \Magento\Catalog\Model\ProductTypes\ConfigInterface $productTypeConfig,
                                \Magento\Framework\Locale\FormatInterface           $localeFormat,
                                \Magento\Customer\Model\Session                     $customerSession,
                                \Magento\Catalog\Api\ProductRepositoryInterface     $productRepository,
                                \Magento\Framework\Pricing\PriceCurrencyInterface   $priceCurrency,
                                \Magento\Catalog\Helper\Image                       $image,
                                \Magento\Framework\Json\Helper\Data                 $serialized,
                                \Psr\Log\LoggerInterface                            $logger,
                                \Magento\Review\Model\ResourceModel\Review\CollectionFactory $reviewCollection,
                                \Magento\Review\Model\ResourceModel\Review\Summary\CollectionFactory $summaryCollection,
                                array                                               $data = [])
    {
        $this->imageHelper = $image;
        $this->serialized = $serialized;
        $this->logger = $logger;
        $this->reviewCollection = $reviewCollection;
        $this->summaryCollection = $summaryCollection;
        parent::__construct($context, $urlEncoder, $jsonEncoder, $string, $productHelper, $productTypeConfig, $localeFormat, $customerSession, $productRepository, $priceCurrency, $data);
    }


    /**
     * @return \Magento\Catalog\Model\Product
     */
    public function getProduct()
    {
        return parent::getProduct(); // TODO: Change the autogenerated stub
    }


    /**
     * @return string
     */
    private function getProductImageUrl()
    {
        //$url = "https://ferreteriaprat.cl/media/catalog/product/s/p/spr022630_sx7wjvipauscymta.jpg?optimize=medium&bg-color=255,255,255&fit=bounds&height=700&width=700&canvas=700:700";
        $url = $this->imageHelper->init($this->getProduct(), 'product_thumbnail_image')->getUrl();
        $list = explode('?', $url);
        if (count($list) > 1) {
            return $list[0];
        }
        return $url;
    }


    /**
     * @return string
     */
    private function getDescription()
    {
        $desciption = $this->getProduct()->getDescription();
        if ($desciption != null) {
            return $desciption;
        }
        return $this->getProduct()->getName();
    }


    public function getDataToArray()
    {
        return [
            "structured" => [
                "@context" => "https://schema.org",
                "@type" => "Product",
                "name" => $this->getProduct()->getName(),
                "image" => $this->getProductImageUrl(),
                "description" => $this->getDescription(),
                "brand" => [
                    "@type" => "Brand",
                    "name" => "Llantas del Pacífico",

                ],
                "SKU" => $this->getProduct()->getSku(),
                "offers" => [
                    "@type" => "Offer",
                    "url" => $this->getProduct()->getProductUrl(),
                    "priceCurrency" => "CLP",
                    "price" => number_format($this->getProduct()->getFinalPrice(), 0, ',', '.'),
                    "availability" => "https://schema.org/InStock",
                    "itemCondition" => "https://schema.org/NewCondition"
                ],

                /*"aggregateRating" => [
                    "@type"=> "AggregateRating",
                    //"ratingValue"=> "5",
                    //"bestRating"=> "5",
                    //"worstRating"=> "5",
                    "ratingCount"=> $this->getReviewAtt('count'),
                    "reviewCount"=> $this->getReviewAtt('count')
                ],*/
                /* "review" => [
                   "@type"=> "Review",
                   "name"=> $this->getReviewAtt('title'),
                   "reviewBody"=> $this->getReviewAtt('detail'),
                   //"reviewRating"=> [
                   //  "@type"=> "Rating",
                     //"ratingValue"=> "5",
                     //"bestRating"=> "5",
                     //"worstRating"=> "5"
                   //],
                   "datePublished"=> substr($this->getReviewAtt('created_at'),0,10),
                   "author"=> ["@type"=> "Person", "name"=> $this->getReviewAtt('nickname')]
                   ]*/
            ],
            "removesc" => true,
            "jsonStringify" => true
            
        ];
    }


    /**
     * @param array $formData
     * @return string
     */
    public function serializedFormData($formData)
    {
        return $this->serialized->jsonEncode($formData);
    }

    public function getReviewAtt($att){
        $review = $this->reviewCollection;
        $collectionReview = $review->create()
            ->addStatusFilter(\Magento\Review\Model\Review::STATUS_APPROVED)
            ->addEntityFilter('product',$this->getProduct()->getId())
            ->addReviewsTotalCount()
            ;
        if ($att =='count') {
            return strval(count($collectionReview->getData()));
        }
        if (!empty($collectionReview->getData())) {
            return $collectionReview->getData()[0][$att];
        } else {
            return "";
        }        

    }
}
