<?php

namespace Ymina\WarehouseStock\Model;

class ManagerIntegration
{
    private $soapUrl;
    private $soapClient;
    private $wsCode;
    private $logger;

    public function __construct(
        \Psr\Log\LoggerInterface $logger // Inyecta un logger para manejar errores
    )
    {
        $this->soapUrl = 'http://wss.imanager.cl:8081/Manager/ws/wsllantas4/wsComando.asmx?WSDL';
        $this->wsCode = '2F6QBXkxMb';
        $this->logger = $logger;
        try {
            $this->soapClient = new \SoapClient($this->soapUrl);
        } catch (\Exception $e) {
            $this->notifyError($e->getMessage());
        }
    }

    public function getWholesaleCustomers()
    {
        try {
            $response = $this->soapClient->Comando02ClientesMayoristas(
                [
                    'Codigo' => $this->wsCode,
                    'NUM_PAGINA' => '1',
                    'TAM_PAGINA' => '8000'
                ]
            );
        } catch (\SoapFault $e) {
            $this->notifyError($e->getMessage());
        }

        return $this->handleResponse($response->Comando02ClientesMayoristasResult);
    }

    public function getWarehouseStocks()
    {
        $response = "";
        try {
            $response = $this->soapClient->Comando01StockPorBodegas(
                [
                    'Codigo' => '2F6QBXkxMb',
                ]
            );
        } catch (\SoapFault $e) {
            $this->notifyError($e->getMessage());
        }

        return $this->handleResponse($response->Comando01StockPorBodegasResult);
    }

    private function notifyError($error)
    {
        // Log the error instead of stopping execution
        $this->logger->error($error);
    }

    private function handleResponse($result)
    {
        if ($result->EsError) {
            $this->notifyError($result->ErrorDescripcion);
            return false; // Return false or throw an exception if you prefer
        }

        return $this->parseResponse($result->Contenido->any);
    }

    private function parseResponse($responseContent)
    {
        $responseContent = '<result>' . $responseContent . '</result>';
        $sxe = new \SimpleXMLElement($responseContent);
        $sxe->registerXPathNamespace('d', 'urn:schemas-microsoft-com:xml-diffgram-v1');
        $result = $sxe->xpath("//NewDataSet")[0];

        // Convertir el resultado SimpleXMLElement en un array
        $arrayResult = [];
        foreach ($result->children() as $child) {
            $arrayResult[] = json_decode(json_encode($child), true);
        }

        return $arrayResult;
    }

}
