<?php

namespace Ymina\WarehouseStock\Model;

class WarehouseStockManager
{
    protected \Magento\Framework\App\ResourceConnection $resourceConnection;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $resourceConnection
    )
    {
        $this->resourceConnection = $resourceConnection;
    }

    public function updateStockData($data): string
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $connection->getTableName('warehouse_stock');

        // Preparing the insert query
        $sqlInsert = 'REPLACE INTO ' . $tableName . ' (sku, name, type, rim, brand, stock) VALUES ';
        $first = true;
        foreach ($data as $wstock) {
            if (!$first) {
                $sqlInsert .= "\n,";
            } else {
                $first = false;
            }
            $sqlInsert .= $this->prepareValues($wstock);
        }
        $sqlInsert .= ';';

        // Executing the query
        try {
            $connection->query($sqlInsert);
            return "Importados " . count($data) . " registros.\n";
        } catch (\Exception $e) {
            throw $e; // Handle exception as needed
        }
    }

    private function prepareValues($wstock): string
    {
        try {
            $codigo = trim((string)$wstock['CODIGO']);
            $nombre = trim((string)$wstock['NOMBRE']);
            $tipo = is_array($wstock['Tipo']) ? 'NULL' : trim((string)$wstock['Tipo']);
            $aro = is_array($wstock['Aro']) ? 'NULL' : trim((string)$wstock['Aro']);
            $marca = is_array($wstock['Marca']) ? 'NULL' : trim((string)$wstock['Marca']);

            unset($wstock['CODIGO'], $wstock['NOMBRE'], $wstock['Tipo'], $wstock['Aro'], $wstock['Marca']);
            $jsonStock = array_filter($wstock);
            $jsonStock = json_encode($jsonStock, JSON_UNESCAPED_UNICODE);
            return "('{$codigo}', '{$nombre}', '{$tipo}', '{$aro}', '{$marca}', '{$jsonStock}')";
        } catch (\LogicException $exception) {
            return "";
        }
    }

    /**
     * @param $codigo
     * @return mixed|null
     */
    public function findByCode($codigo)
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $connection->getTableName('warehouse_stock');

        $select = $connection->select()
            ->from($tableName)
            ->where('sku = ?', $codigo);

        $result = $connection->fetchRow($select);

        return $result ? $result : null;
    }
}
