require([
    'jquery',
    'owlcarousel',
    'domReady!'
], function ($) {
    $(document).ready(function () {

        /** Show sections-overlay with click in toggle-nav */
        $('.burger').on('click', function (e) {
            e.preventDefault();
            if ($(window).width() >= 1024) {
                if ($('html').hasClass('nav-before-open nav-open')) {
                    $('.sections-overlay').hide()
                } else {
                    $('.sections-overlay').show()
                }
            }
        })

        /** Close menu/custom search with sections-overlay */
        $('.sections-overlay').on('click', function (e) {
            e.preventDefault();
            //if ($(window).width() >= 1024) {
            $(this).hide()
            $('html').removeClass('nav-before-open nav-open custom-search-active')
            $('.filter-container').removeClass('active')
            //}
        })

        /**
         * Validation Rut
         * **/


        function validarRutEvent() {
            $(document).on('input keydown', 'input[name="custom_attributes[address_rut]"]', function (e) {
                let value = $(this).val().replace(/\./g, '').replace('-', '');
                var key = e.keyCode || e.charCode;
                if (key == 8 || key == 46) {
                    value = value.substr(0, value.length - 1);
                    e.preventDefault();
                }
                $(this).val(validateAndFormat(value));
            });
        }

        validarRutEvent();
        validarRutRegisterEvent();

        function validarRutRegisterEvent() {
            $(document).on('keypress', 'input[name="rut"]', function (e) {
                let value = $(this).val().replace(/\./g, '').replace('-', '');
                var key = e.keyCode || e.charCode;
                if (key == 8 || key == 46) {
                    value = value.substr(0, value.length - 1);
                    e.preventDefault();
                }
                $(this).val(validateAndFormat(value));

                if (!isValidRut($(this).val())) {
                    if (!$('#rut-error-msg').length) {
                        $(this).parent('.control').append('<div for="rut" generated="true" class="mage-error" id="rut-error-msg"> Digite un RUT valido en este campo. </div>').show();
                    }
                } else {
                    $('#rut-error-msg').remove();
                }
            });
            $(document).on('blur', 'input[name="rut"]', function () {
                $('input[name="rut"]').trigger('keypress');
            });
        }

        function validateAndFormat(value) {
            if (value.match(/^(\d{2})(\d{3}){2}(\w{1})$/)) {
                value = value.replace(/^(\d{2})(\d{3})(\d{3})(\w{1})$/, '$1.$2.$3-$4');
            } else if (value.match(/^(\d)(\d{3}){2}(\w{0,1})$/)) {
                value = value.replace(/^(\d)(\d{3})(\d{3})(\w{0,1})$/, '$1.$2.$3-$4');
            } else if (value.match(/^(\d)(\d{3})(\d{0,2})$/)) {
                value = value.replace(/^(\d)(\d{3})(\d{0,2})$/, '$1.$2.$3');
            } else if (value.match(/^(\d)(\d{0,2})$/)) {
                value = value.replace(/^(\d)(\d{0,2})$/, '$1.$2');
            }
            return value;
        }

        function isValidRut(rutCompleto) {
            rutCompleto = rutCompleto.replace(/\./g, '');
            var tmp = rutCompleto.split('-');
            var digv = tmp[1];
            var rut = tmp[0];
            if (rut.length < 7 || rut.length > 8) {
                return false
            }
            if (digv == 'K') digv = 'k';
            return (rutDv(rut) == digv);
        }

        function rutDv(T) {
            var M = 0,
                S = 1;
            for (; T; T = Math.floor(T / 10))
                S = (S + T % 10 * (9 - M++ % 6)) % 11;
            return S ? S - 1 : 'k';
        }

        /** Moving top-links class*/
        $('.header.links > li:eq(0) > a').removeClass('link my-account-link').parent().addClass('link my-account-link')
        $('.header.links > li:eq(2) > a').removeClass('link register-link').parent().addClass('link register-link')

        /** Carousel huincha header */
        $('.huincha-header').owlCarousel({
            loop: true,
            margin: 10,
            nav: true,
            navText: [
                "<i class='fa-solid fa-angle-left'></i>",
                "<i class='fa-solid fa-angle-right'></i>"
            ],
            autoplay: true,
            autoplayHoverPause: true,
            responsiveClass: true,
            responsive: {
                0: {
                    items: 1
                },
                600: {
                    items: 1
                },
                1000: {
                    items: 1
                }
            }
        });

        /** Carousel calugas home */
        $('.calugas-home').removeClass('owl-carousel');
        setTimeout(function () {
            $('.pagebuilder-column-group.calugas-home > .pagebuilder-column-line').addClass('owl-carousel').css({'visibility': 'visible'});
            setTimeout(function () {
                $('.pagebuilder-column-group.calugas-home > .pagebuilder-column-line.owl-carousel').owlCarousel({
                    loop: false,
                    margin: 30,
                    nav: false,
                    dots: false,
                    navText: [
                        "<i class='fa-solid fa-angle-left'></i>",
                        "<i class='fa-solid fa-angle-right'></i>"
                    ],
                    autoplay: false,
                    autoplayHoverPause: false,
                    responsiveClass: true,
                    responsive: {
                        0: {
                            margin: 10,
                            items: 2,
                            loop: true,
                            autoplay: true,
                            autoplayHoverPause: true,
                            dots: true
                        },
                        900: {
                            margin: 10,
                            items: 3,
                            dots: true
                        },
                        1200: {
                            items: 4
                        }
                    }
                });
            }, 1000);
        }, 100);

        /** Carousel brands */
        $('.brands-carousel').owlCarousel({
            loop: true,
            margin: 60,
            nav: false,
            navText: [
                "<i class='fa-solid fa-angle-left'></i>",
                "<i class='fa-solid fa-angle-right'></i>"
            ],
            autoplay: true,
            autoplayHoverPause: true,
            responsiveClass: true,
            responsive: {
                0: {
                    items: 2,
                    margin: 30
                },
                800: {
                    items: 4,
                    margin: 30
                },
                1000: {
                    items: 7
                }
            }
        });

        /** Back to top btn */
        var btn = $('#backtotop');

        $(window).scroll(function () {
            if ($(window).width() <= 1024) {
                if ($(window).scrollTop() > 30) {
                    $('.page-wrapper').addClass('fixed-active');
                    $('.page-header').addClass('fixed');
                    $('.header.top').hide();
                } else {
                    $('.page-wrapper').removeClass('fixed-active');
                    $('.page-header').removeClass('fixed');
                    $('.header.top').show();
                }
            }

            if ($(window).scrollTop() > 300) {
                btn.addClass('show');
            } else {
                btn.removeClass('show');
            }
        });

        btn.on('click', function (e) {
            e.preventDefault();
            $('html, body').animate({scrollTop: 0}, '300');
        });

        // $(window).on('resize', function(){
        // 	var win = $(this)
        // 	if(win.width() <= 1024) {}else{}
        // })

        /** Accordion footer column mobile */
        $("footer .pagebuilder-column.accordion h2").on("click", function (e) {
            e.preventDefault();
            if ($(this).hasClass("active")) {
                $(this).removeClass("active")
                $(this).siblings("[data-content-type='text']").removeClass("active")
            } else {
                $("footer .pagebuilder-column.accordion h2").removeClass("active")
                $(this).addClass("active")
                $("footer .pagebuilder-column.accordion [data-content-type='text']").removeClass("active")
                $(this).siblings("[data-content-type='text']").addClass("active")
            }
        });

    });
});

// require([
//     'jquery',
//     'mage/url',
//     'slick'
// ], function ($, url, slick) {
// $('.huincha-header').slick({
//     infinite: true,
//     slidesToShow: 1,
//     slidesToScroll: 1,
//     dots: false,
//     arrows: true,
//     autoplay: true,
//     autoplaySpeed: 1000,
//     responsive: [
//         {
//             breakpoint: 1024,
//             settings: {
//                 slidesToShow: 1,
//                 slidesToScroll: 1
//             }
//         }
//     ]
// });
// });
